#include <stdlib.h>
#include <string.h>
#include "ial.h"
#include "incubus.h"

extern IAL_HEAD ial_head;
extern dyn_AppendElement *Tcl_AppendElement;

IAL_HEAD find_ial_interp(void *interp)
{
	IAL_HEAD dummy;

	for(dummy = ial_head; dummy; dummy = dummy->next)
	{
		if(interp == dummy->interp)
			return dummy;
	}

	return NULL;
}

IAL find_ial_user(IAL_HEAD head, char *nick)
{
	IAL dummy;

	if(!head)
		return NULL;

	for(dummy = head->ial; dummy; dummy = dummy->next)
	{
		if(!stricmp(dummy->nick, nick))
			return dummy;
	}

	return NULL;
}

IAL_HEAD add_ial_interp(void *interp)
{
	IAL_HEAD dummy;

	if((dummy = find_ial_interp(interp)) != NULL)
		return dummy;

	if((dummy = (IAL_HEAD)malloc(sizeof(struct ial_headstruct))) == NULL)
		return NULL;

	dummy->interp = interp;
	dummy->ial = NULL;
	dummy->next = ial_head;
	ial_head = dummy;
	return dummy;
}

IAL_CHAN find_user_channel(IAL user, char *channel)
{
	IAL_CHAN dummy;

	if(!user)
		return NULL;

	for(dummy = user->channels; dummy; dummy = dummy->next)
	{
		if(!stricmp(channel, dummy->channel))
			return dummy;
	}

	return NULL;
}

char *ial_address(void *interp, char *nick)
{
	static char *retval = NULL;
	short int len1, len2;
	IAL user;


	if((user = find_ial_user(find_ial_interp(interp), nick)) == NULL)
		return NULL;

	if(retval)
		free(retval);

	len1 = strlen(user->nick);
	len2 = strlen(user->userhost);

	if((retval = (char *)malloc(len1 + len2 + 2)) == NULL)
		return NULL;

	strcpy(retval, user->nick);
	strcat(retval, "!");
	strcat(retval, user->userhost);

	return retval;
}

int add_to_ial(void *interp, char *nick, char *uhost, char *channel)
{
	IAL_CHAN channels;
	IAL_HEAD head;
	IAL user;
	
	if((head = find_ial_interp(interp)) == NULL)
		head = add_ial_interp(interp);
	user = find_ial_user(head, nick);

	if(user == NULL)
	{
		if((user = (IAL)malloc(sizeof(struct ial_struct))) == NULL)
			return 0;

		user->nick = strdup(nick);
		user->userhost = strdup(uhost);
		user->channels = NULL;
		user->next = head->ial;
		head->ial = user;
		if((channels = (IAL_CHAN)malloc(sizeof(struct ial_chanstruct))) == NULL)
		{
			return 0;
		}
		channels->channel = strdup(channel);
		channels->next = NULL;
		user->channels = channels;
		return 1;
	}
	else
	{
		for(channels = user->channels; channels; channels = channels->next)
		{
			if(!stricmp(channels->channel, channel))
				return 0;
		}
		if((channels = (IAL_CHAN)malloc(sizeof(struct ial_chanstruct))) == NULL)
		{
			return 0;
		}
		channels->channel = strdup(channel);
		channels->next = user->channels;
		user->channels = channels;
		return 1;
	}
	return 1;
}

int remove_channel_from_ial(void *interp, char *channel)
{
	IAL_HEAD head;
	IAL_CHAN chan, lastchan, tempchan;
	IAL user, lastuser = NULL, tempuser;
	int retval = 0;

	if((head = find_ial_interp(interp)) == NULL)
		return retval;

	for(user = head->ial; user;)
	{
		lastchan = NULL;
		chan = user->channels;
		while(chan)
		{
			if(!stricmp(chan->channel, channel))
			{
				retval++;
				free(chan->channel);

				if(lastchan)
					lastchan->next = chan->next;
				else
					user->channels = chan->next;

				free(chan);
				chan = NULL;
			}
			else
			{
				lastchan = chan;
				chan = chan->next;
			}
		}

		if(!user->channels)
		{
			free(user->nick);
			free(user->userhost);

			if(lastuser)
			{
				lastuser->next = user->next;
			}
			else
			{
				head->ial = user->next;
			}
			tempuser = user;
			user = user->next;
			free(tempuser);
		}
		else
		{
			lastuser = user;
			user = user->next;
		}
	}
	return retval;
}

int remove_channel_from_ial_nick(void *interp, char *nick, char *channel)
{
	IAL_CHAN chan, lastchan = NULL;
	IAL user;

	if((user = find_ial_user(find_ial_interp(interp), nick)) == NULL)
		return 0;

	for(chan = user->channels; chan; lastchan = chan, chan = chan->next)
	{
		if(!stricmp(chan->channel, channel))
		{
			free(chan->channel);
			
			if(lastchan)
				lastchan->next = chan->next;
			else
				user->channels = chan->next;

			free(chan);
			if(user->channels == NULL)
				delete_ial_user(interp, nick);
			return 1;
		}
	}
	return 0;
}

int change_ial_nick(void *interp, char *oldnick, char *newnick)
{
	IAL user;

	if((user = find_ial_user(find_ial_interp(interp), oldnick)) == NULL)
		return 0;

	free(user->nick);
	user->nick = strdup(newnick);
	return 1;
}

int clear_ial(void *interp)
{
	IAL_HEAD head;
	IAL_CHAN chan, lastchan;
	IAL user, lastuser;

	if((head = find_ial_interp(interp)) == NULL)
		return 0;

	for(user = head->ial; user; lastuser = user, user = user->next, free(lastuser))
	{
		for(chan = user->channels; chan; lastchan = chan, chan = chan->next, free(lastchan))
		{
			free(chan->channel);
		}

		free(user->nick);
		free(user->userhost);
	}
	head->ial = NULL;
	return 1;
}

int delete_ial_user(void *interp, char *nick)
{
	IAL_HEAD head;
	IAL_CHAN chan, lastchan;
	IAL user, lastuser = NULL;

	if((head = find_ial_interp(interp)) == NULL)
		return 0;

	for(user = head->ial; user; user = user->next)
	{
		if(!stricmp(user->nick, nick))
		{
			for(chan = user->channels; chan; lastchan = chan, chan = chan->next, free(lastchan))
			{
				free(chan->channel);
			}

			free(user->nick);
			free(user->userhost);
	
			if(lastuser)
				lastuser->next = user->next;
			else
				head->ial = user->next;

			free(user);
			return 1;
		}
		lastuser = user;
	}
	return 0;
}

int delete_ial_interp(void *interp)
{
	IAL_HEAD head, lasthead = NULL;

	clear_ial(interp);
	for(head = ial_head; head; head = head->next)
	{
		if(head->interp == interp)
		{
			if(lasthead)
				lasthead->next = head->next;
			else
				ial_head = head->next;

			free(head);
			return 1;
		}
		lasthead = head;
	}
	return 0;
}

int ial_ison(void *interp, char *nick, char *channel)
{
	IAL user;
	IAL_CHAN chan;


	if((chan = find_user_channel(find_ial_user(find_ial_interp(interp), nick), channel)) == NULL)
		return 0;

	return 1;
}
void ial_nicks(void *interp)
{
	IAL_HEAD head;
	IAL user;

	if((head = find_ial_interp(interp)) == NULL)
		return;

	for(user = head->ial; user; user = user->next)
	{
		(*Tcl_AppendElement)(interp, user->nick);
	}
}

char *ial_userhost(void *interp, char *nick)
{
	IAL user;


	if((user = find_ial_user(find_ial_interp(interp), nick)) == NULL)
		return NULL;

	return user->userhost;
}

void ial_userchannels(void *interp, char *nick)
{
	IAL_CHAN chan;
	IAL user;

	if((user = find_ial_user(find_ial_interp(interp), nick)) == NULL)
		return;

	for(chan = user->channels; chan; chan = chan->next)
	{
		(*Tcl_AppendElement)(interp, chan->channel);
	}
	return;
}

