#ifndef __incubus_h
#define __incubus_h

/* return values for TCL functions; there are three others, but
   these should suffice. */
#define TCL_OK 0
#define TCL_ERROR 1

/* Tcl_CmdProc is a typedef describing the function TCL expects to call
   to execute a command */
typedef int Tcl_CmdProc(void *cd, void *interp, int argc, char *argv[]);

/* typedefs for convenience: dyn_CreateCommand and dyn_AppendResult are
   declarations for the two functions from xircntcl.dll that we'll need. */
/* This is Tcl_CreateCommand(Tcl_Interp *interp, const char *cmdName,
   Tcl_CmdProc *proc, ClientData clientdata, Tcl_CmdDeleteProc *deleteProc);
   Since interp, clientdata, and deleteProc are of no interest, we'll refer
   to them as void * (the sizes match) so as to avoid defining them. */
typedef void dyn_CreateCommand(void*, const char*, Tcl_CmdProc*, void*, void*);
/* Tcl_AppendResult(Tcl_Interp *interp, ...); again, interp is of little
   interest.  IMPORTANT: this command expects a list of char*'s to append to
   the command's result (actually the interpreter's result, but why quibble)
   followed by a NULL as the last argument.  If you forget to close it with
   a NULL, bad things will happen. */
typedef int dyn_AppendResult(void*, ...);

char *Tcl_Merge(int argc, char **argv);

typedef void dyn_AppendElement(void *, char *);
/*    Tcl_Interp *interp;  Interpreter whose result is to be
                           extended.
      char *string;        String to convert to list element and
                           add to result. */
typedef char *dyn_GetVar(void *, char *, int);
/*	Tcl_Interp *interp;			* Command interpreter in which varName is
								* to be looked up.
	char *varName;				* Name of a variable in interp.
	int flags;					* OR-ed combination of TCL_GLOBAL_ONLY
								* or TCL_LEAVE_ERR_MSG bits. */

typedef int dyn_Eval(void *, char *);
/*	Tcl_Interp *interp;			* Token for command interpreter (returned
								* by a previous call to Tcl_CreateInterp).
	char *cmd;					* Pointer to TCL command to interpret. */


#define TCL_GLOBAL_ONLY         1
#define TCL_APPEND_VALUE        2
#define TCL_LIST_ELEMENT        4
#define TCL_TRACE_READS         0x10
#define TCL_TRACE_WRITES        0x20
#define TCL_TRACE_UNSETS        0x40
#define TCL_TRACE_DESTROYED     0x80
#define TCL_INTERP_DESTROYED    0x100
#define TCL_LEAVE_ERR_MSG       0x200

#define TCL_DONT_USE_BRACES	1
#define USE_BRACES			2
#define BRACES_UNMATCHED	4

#endif /* __incubus_h */

