#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "servers.h"
#include "misc.h"

extern SERVER servers;
extern int serversread;

SERVER addserver(char *network_type, char *servername)
{
	SERVER dummy;
	SERVER newserver;

	for(dummy = servers; dummy; dummy = dummy->next)
	{
		if(!stricmp(dummy->name, servername) && 
			!stricmp(dummy->network, network_type))
		{
			return dummy;
		}
	}

	if((newserver = malloc(sizeof(struct serverstruct))) == NULL)
		return NULL;

	newserver->network = strdup(network_type);
	newserver->name = strdup(servername);
	newserver->ports = NULL;
	newserver->next = servers;
	servers = newserver;
	serversread = 1;

	return newserver;
}

int addport(SERVER server, int port)
{
	PORT dummy;
	PORT newport;

	for(dummy = server->ports; dummy; dummy = dummy->next)
		if(dummy->port == port)
			return 0;

	if((newport = malloc(sizeof(struct portstruct))) == NULL)
		return 0;

	newport->port = port;
	newport->next = server->ports;
	server->ports = newport;

	return 1;
}

void parse_serverline(char *line)
{
	char *network_type;
	char *servername;
	char *ports;
	char *port;
	SERVER server;

	get_token(&line, "=");
	network_type = get_token(&line, ":");
	get_token(&line, ":");
	servername = get_token(&line, ":");
	ports = get_token(&line, ":G");

	if(network_type == NULL || servername == NULL || ports == NULL)
		return;

	if((server = addserver(network_type, servername)) == NULL)
		return;

	while((port = get_token(&ports, ",")) != NULL)
		addport(server, atoi(port));
}

int readservers(char *filename)
{
	FILE *infile;
	char buf[1024];

	if((infile = fopen(filename, "r")) == NULL)
		return -1;

	while(!feof(infile))
	{
		fgets(buf, 1024, infile);
		parse_serverline(buf);
	}

	fclose(infile);
	return 1;
}

void clear_servers(void)
{
	SERVER server, lastserver;
	PORT port, lastport;

	for(server = servers; server; server = server->next, free(lastserver))
	{
		for(port = server->ports; port; port = port->next, free(lastport));
			lastport = port;

		free(server->network);
		free(server->name);
		lastserver = server;
	}
}

char *get_servertype(char *servername, int serverport)
{
	SERVER server;
	PORT port;

	for(server = servers; server; server = server->next)
	{
		if(!stricmp(server->name, servername))
			for(port = server->ports; port; port = port->next)
				if(port->port == serverport)
					return server->network;
	}

	return "";
}
