#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "incubus.h"
#include "users.h"
#include "match.h"
#include "misc.h"

extern int usersread;
extern USER users;
extern USER bots;
extern dyn_AppendElement *Tcl_AppendElement;

void clear_all_users(void)
{
	USER dummy, last;

	for(dummy = users; dummy; last = dummy, dummy = dummy->next, free(last))
	{
		free(dummy->network);
		free(dummy->mask);
		free(dummy->password);
		last = dummy;
	}

	users = NULL;

	for(dummy = users; dummy; last = dummy, dummy = dummy->next, free(last))
	{
		free(dummy->network);
		free(dummy->mask);
		free(dummy->password);
		last = dummy;
	}

	bots = NULL;

	return;
}

void adduser(char *network, char *mask, int flags, char *password)
{
	USER dummy;
	USER newuser;

	if(password)
		dummy = bots;
	else
		dummy = users;

	for(; dummy; dummy = dummy->next)
	{
		if(!stricmp(dummy->mask, mask) && 
			!stricmp(dummy->network, network))
		{
			if(dummy->flags != flags)
				dummy->flags = flags;
			if(password)
			{
				free(dummy->password);
				dummy->password = strdup(password);
			}
			return;
		}
	}

	if((newuser = malloc(sizeof(struct userstruct))) == NULL)
		return;

	newuser->network = strdup(network);
	newuser->mask = strdup(mask);
	newuser->flags = flags;
	if(password)
	{
		newuser->password = strdup(password);
		newuser->next = bots;
		bots = newuser;
	}
	else
	{
		newuser->next = users;
		users = newuser;
	}
	usersread = 1;
}

void parseline(char *line)
{
	char *token;
	int  type;
	char *network;
	char *mask;
	int  flags;
	char *password = NULL;

	token = get_token(&line, " ");
	if(*token == '#' || token == NULL)
		return;
	if(!stricmp(token, "user"))
		type = 1;
	else if(!stricmp(token, "bot"))
		type = 2;
	else
		return;

	network = get_token(&line, " ");
	mask = get_token(&line, " ");
	token = get_token(&line, " ");
	if(network == NULL || mask == NULL || token == NULL)
		return;
	flags = atoi(token);
	if(type == 2)
	{
		char tmp[4];

		tmp[0] = ' ';
		tmp[1] = 13;
		tmp[2] = 10;
		tmp[3] = '\0';
		password = get_token(&line, tmp);
		if(password == NULL)
			return;
		adduser(network, mask, flags, password);
	}
	else
		adduser(network, mask, flags, NULL);
}

int read_user_file(char *filename)
{
	FILE *infile;
	char buf[1024];

	if(usersread)
		return 0;
	if ((infile = fopen(filename, "r")) == NULL)
		return -1;

	while(!feof(infile))
	{
		fgets(buf, 1024, infile);
		parseline(buf);
	}

	fclose(infile);
	return 1;
}
int write_user_file(char *filename)
{
	FILE *outfile;
	USER dummy;

	if(!usersread)
		return 0;
	if ((outfile = fopen(filename, "w")) == NULL)
		return -1;

	for(dummy = users; dummy; dummy = dummy->next)
	{
		fprintf(outfile, "user %s %s %d\n", dummy->network, dummy->mask, dummy->flags);
	}

	for(dummy = bots; dummy; dummy = dummy->next)
	{
		fprintf(outfile, "bot  %s %s %d %s\n", dummy->network, dummy->mask, dummy->flags, dummy->password);
	}

	fclose(outfile);
	return 1;
}

int isbot(char *network, char *address)
{
	USER dummy;

	for(dummy = bots; dummy; dummy = dummy->next)
	{
		if(!stricmp(network, dummy->network) && !_match(dummy->mask, address))
			return dummy->flags;
	}
	return 0;
}

int isuser(char *network, char *address)
{
	USER dummy;

	for(dummy = users; dummy; dummy = dummy->next)
	{
		if(!stricmp(network, dummy->network) && !_match(dummy->mask, address))
			return dummy->flags;
	}
	return 0;
}

char *getpass(char *network, char *address)
{
	USER dummy;

	for(dummy = bots; dummy; dummy = dummy->next)
	{
		if(!stricmp(network, dummy->network) && !_match(dummy->mask, address))
			return dummy->password;
	}
	return "";
}

int totalusers(char *network)
{
	USER dummy;
	int total = 0;

	for(dummy = users; dummy; dummy = dummy->next)
	{
		if(!_match(network, dummy->network))
			total++;
	}
	return total;
}

int totalbots(char *network)
{
	USER dummy;
	int total = 0;

	for(dummy = bots; dummy; dummy = dummy->next)
	{
		if(!_match(network, dummy->network))
			total++;
	}
	return total;
}

char *botindex(char *network, int index)
{
	USER dummy;
	int total = 0;

	for(dummy = bots; dummy; dummy = dummy->next)
	{
		if(!_match(network, dummy->network))
			if(total == index)
				return dummy->mask;
			else
				total++;
	}
	return "";
}

char *userindex(char *network, int index)
{
	USER dummy;
	int total = 0;

	for(dummy = users; dummy; dummy = dummy->next)
	{
		if(!_match(network, dummy->network))
			if(total == index)
				return dummy->mask;
			else
				total++;
	}
	return "";
}

int deluser(char *network, char *mask, int type)
{
	USER dummy;
	USER last = NULL;

	if(type == 1)
		dummy = users;
	else
		dummy = bots;

	for(; dummy; dummy = dummy->next)
	{
		if(!stricmp(network, dummy->network))
		{
			if(!stricmp(dummy->mask, mask))
			{
				if(!last)
				{
					if(type == 1)
						users = dummy->next;
					else
					{
						bots = dummy->next;
						free(dummy->password);
					}
					free(dummy->network);
					free(dummy->mask);
					free(dummy);
				}
				else
				{
					last->next = dummy->next;
					free(dummy->network);
					free(dummy->mask);
					if(type == 1)
						free(dummy->password);
					free(dummy);
				}
				return 1;
			}
		}
		last = dummy;
	}
	return 0;
}

void get_userlist(void *interp, USER list, char *network, char *mask)
{
	USER dummy;
	char *argv[4];
	char temp[16];
	char *result = NULL;

	for(dummy = list; dummy; dummy = dummy->next)
	{
		if(!_match(network, dummy->network) && !_match(mask, dummy->mask))
		{
			argv[0] = dummy->network;
			argv[1] = dummy->mask;
			argv[2] = itoa(dummy->flags, temp, 10);
			if(list != users)
			{
				argv[3] = dummy->password;
				result = (*Tcl_Merge)(4, argv);
			}
			else
				result = (*Tcl_Merge)(3, argv);

			(*Tcl_AppendElement)(interp, result);
			free(result);
			result = NULL;
		}
	}
}
